/***********************************************************
 * $Id: CompressionHelper.java 511 2011-08-15 16:49:51Z util $
 * 
 * HTTP Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 17.09.2010
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.http;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.zip.DeflaterOutputStream;
import java.util.zip.GZIPOutputStream;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * This class provides static functionality for handling compression algorithms
 * controlled through the <code>Content-Coding</code> header field.
 */
public abstract class CompressionHelper
{
    
    /**
     * A default set of supported encodings.
     */
    public static final String[] DEFAULT_SUPPORTED_ENCODINGS = new String[] {"gzip","deflate"};
    
    /**
     * 
     * @param acceptEncoding The content of an Accept-Encoding header (see section 14.3 of RFC2616).
     * @param supportedEncodings A list of supported encodings to choose from. This list should
     *                           not contain the <code>identity</code> encoding, because
     *                           noop compression is selected through a <code>null</code>
     *                           encoding value.
     * @return The encoding selected to use for the given browser of <code>null</code>, if the response
     *         should not be compressed.
     */
    public static String selectCompressionCoding(String acceptEncoding, String[] supportedEncodings) {
        
    	return AcceptHelper.selectAcceptItem(acceptEncoding,supportedEncodings);
    }
    
    /**
     * Parse the <code>Accept-Encoding</code> header of the request and set up a possibly compressed
     * output stream to write to. If the output is chosen to be compressed, an appropriate
     * <code>Content-Encoding</code> is set on the passed HTTP response.
     * 
     * @param req The HTTP request to examine.
     * @param resp The HTTP response.
     * @return A compressed or uncompressed stream based on the <code>Accept-Encoding</code>
     *         header of the client. The returned stream should be finished
     *         after writing the whole response using {@link #finishOutputStream(OutputStream)}.
     * @throws IOException Upon errors when setting response headers or setting up streams.
     */
    public static OutputStream setupResponseOutputStream(HttpServletRequest req, HttpServletResponse resp) throws IOException {
        
        String acceptEncoding = req.getHeader("Accept-Encoding");
        String selectedEncoding = selectCompressionCoding(acceptEncoding,DEFAULT_SUPPORTED_ENCODINGS); 
            
        if (selectedEncoding == null)
            return resp.getOutputStream();
        else {
            
            resp.setHeader("Content-Encoding",selectedEncoding);
            
            if ("deflate".equals(selectedEncoding)) {
        
                return new DeflaterOutputStream(resp.getOutputStream());
            }
            else {
                return new GZIPOutputStream(resp.getOutputStream());
            }
        }
    }
    
    /**
     * @param os An output stream previously set up using
     *           {@link #setupResponseOutputStream(HttpServletRequest, HttpServletResponse)}.
     * @return Whether the output stream is compressed or not.
     */
    public static boolean isCompressed(OutputStream os) {
        
        return os instanceof DeflaterOutputStream;
    }
    
    /**
     * Finish writing to a possibly compressed servlet output stream.
     * 
     * @param os An OutputStream returned by {@link #setupResponseOutputStream(HttpServletRequest, HttpServletResponse)}.
     * @throws IOException Upon I/O errors.
     */
    public static void finishOutputStream(OutputStream os) throws IOException {
        
        if (os instanceof DeflaterOutputStream) {
            DeflaterOutputStream dos = (DeflaterOutputStream) os;
            dos.finish();
        }
        else {
            os.flush();
        }
    }

    /**
     * Write an error response to the servlet output stream. The error
     * response may be send in compressed for, if
     * {@link #setupResponseOutputStream(HttpServletRequest, HttpServletResponse)}
     * has been invoked before.
     * 
     * @param resp The servlet response.
     * @param os An output stream previously set up using
     *           {@link #setupResponseOutputStream(HttpServletRequest, HttpServletResponse)}.
     * @param status A HTTP status, which corresponds to the given error situation.
     * @param message An optional error message.
     * @throws IOException Upon errors writing the response to the stream.
     */
    public static void sendError(HttpServletResponse resp, OutputStream os, int status, String message) throws IOException {

    	sendError(resp,os,status,message,null);
    }
    	
    /**
     * Write an error response to the servlet output stream. The error
     * response may be send in compressed for, if
     * {@link #setupResponseOutputStream(HttpServletRequest, HttpServletResponse)}
     * has been invoked before.
     * 
     * @param resp The servlet response.
     * @param os An output stream previously set up using
     *           {@link #setupResponseOutputStream(HttpServletRequest, HttpServletResponse)}.
     * @param status A HTTP status, which corresponds to the given error situation.
     * @param message An optional error message.
     * @param e An optional exception to render.
     * @throws IOException Upon errors writing the response to the stream.
     */
    public static void sendError(HttpServletResponse resp, OutputStream os, int status, String message, Throwable e) throws IOException {
    	
        if (os instanceof DeflaterOutputStream) {
        	
        	if (resp.isCommitted())
        		throw new IllegalStateException();
        	
        	resp.setStatus(status);
        	
        	OutputStreamWriter w = new OutputStreamWriter(os,"UTF-8");
           	PrintWriter pw = new PrintWriter(w);
       	
        	if (message == null)
        		message = "An error occurred while accessing a compressed resource.";
        	
        	resp.setContentType("text/html; charset=UTF-8");
        	
        	pw.write("<html>\n"+
        			"<head>\n"+
        			"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"/>\n"+
        			"<title>Server Error ");
        	
        	pw.print(status);
        	
           	pw.write("</title>\n"+
        			"<head/>\n"+
        			"<body>\n"+
        			"<h2>HTTP ERROR ");
           	
           	pw.print(status);
            
           	pw.write("</h2>\n"+
           			 "<p><![CDATA[");
        	pw.write(message);
         	pw.write("]]></p>\n");
          	
        	if (e != null) {
        		
               	pw.write("<h3>Caused by:</h3>\n"+
               			 "<pre><![CDATA[");

               	e.printStackTrace(pw);
               	
              	pw.write("]]></pre>\n");
        	}
        	
        	pw.write("</body>\n"+
        			 "</html>\n");
        	
        	pw.close();
        	
          	finishOutputStream(os);
        }
        else {
        	resp.sendError(status,message);
        }
    }
}
