/***********************************************************
 * $Id: UrlHelper.java 514 2011-08-19 12:44:14Z util $
 * 
 * HTTP Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 19.08.2011
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.http;

import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URL;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.nio.charset.IllegalCharsetNameException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Static helpers for manipulating URLs and URIs.
 */
public class UrlHelper {

	/**
	 * The name of the UTF-8 encoding for {@link Charset#forName(String)}.
	 */
	public static final String UTF_8 = "UTF-8";
	
	/**
     * Translates a string into <code>application/x-www-form-urlencoded</code>
     * format using UTF-8 encoding as stated in the <a href=
     * "http://www.w3.org/TR/html40/appendix/notes.html#non-ascii-chars">
     * World Wide Web Consortium Recommendation</a>.
     * <br><br>
     * In contrast to <code>URLEncoder.encode(s,"UTF-8")</code> this function
     * does not throw any checked exception.
     *
     * @param   s   <code>String</code> to be translated.
     * @return  the translated <code>String</code>.
     * @see URLEncoder#encode(java.lang.String, java.lang.String)
     */
	public static final String encodeUtf8(String s) {
		try {
			return URLEncoder.encode(s,UTF_8);
		} catch (UnsupportedEncodingException e) {
			throw new IllegalCharsetNameException(UTF_8);
		}
	}
	
    /**
     * Decodes a <code>application/x-www-form-urlencoded</code> string using
     * UTF-8 encoding as stated in the <a href=
     * "http://www.w3.org/TR/html40/appendix/notes.html#non-ascii-chars">
     * World Wide Web Consortium Recommendation</a>.
     * <br><br>
     * In contrast to <code>URLDecoder.decode(s,"UTF-8")</code> this function
     * does not throw any checked exception.
     *
     * @param s the <code>String</code> to decode.
     * @return the newly decoded <code>String</code>
     * @see URLDecoder#decode(String, String)
     */
	public static final String decodeUtf8(String s) {
		try {
			return URLDecoder.decode(s,UTF_8);
		} catch (UnsupportedEncodingException e) {
			throw new IllegalCharsetNameException(UTF_8);
		}
	}
	
	/**
	 * Append additional query parameters to the end of a given URL.
	 * 
	 * @param url The base URL, which my already contain other query parameters.
	 * @param key The name of the parameter.
	 * @param value The value, which will be appended using {@link #encodeUtf8(String)}.
	 * @return The base URL extended by an additional query parameter.
	 */
	public static final String appendQueryParameterToUrl(String url, String key, String value) {
		if (url.contains("?")) {
			return url+"&"+encodeUtf8(key)+"="+encodeUtf8(value);
		} else {
			return url+"?"+encodeUtf8(key)+"="+encodeUtf8(value);
		}
	}

	/**
	 * Append additional query parameters to the end of a given query string.
	 * 
	 * @param q The query part of an URL or URI as returned by {@link URL#getQuery()} or {@link URI#getQuery()}.
	 * @param key The name of the parameter.
	 * @param value The value, which will be appended using {@link #encodeUtf8(String)}.
	 * @return The query extended by an additional query parameter.
	 */
	public static final String appendQueryParameter(String q, String key, String value) {
		if (q != null && q.length() > 0) {
			return q +"&"+encodeUtf8(key)+"="+encodeUtf8(value);
		} else {
			return encodeUtf8(key)+"="+encodeUtf8(value);
		}
	}

	/**
	 * Parse a given query string in <code>application/x-www-form-urlencoded</code>
     * format using UTF-8 encoding.
	 * 
	 * @param q The query part of an URL or URI as returned by {@link URL#getQuery()} or {@link URI#getQuery()}.
	 * @return A map of query parameters with their values. A parameter value
	 *         may be manifold, so the value is represented by a list of strings.
	 */
	public static final Map<String,List<String>> parseQuery(String q) {
		int pos = 0;
		Map<String,List<String>> ret = new HashMap<String,List<String>>();
		
		do {
			int p1 = q.indexOf('=',pos);
			int p2;
			
			String key;
			String value;
			
			if (p1 < 0) {
				p2 = q.indexOf('&',pos);
				
				if (p2 < 0) {
					key = decodeUtf8(q.substring(pos));
					pos = q.length();
				} else {
					key = decodeUtf8(q.substring(pos,p2));
					pos = p2+1;
				}
				value = null;
			} else {
				p2 = q.indexOf('&',pos);

				if (p2 < 0) {
					key = decodeUtf8(q.substring(pos,p1));
					value = decodeUtf8(q.substring(p1+1));
					pos = q.length();
				} else if (p2 < p1) {
					key = decodeUtf8(q.substring(pos,p2));
					value = null;
					pos = p2+1;
				} else {
					key = decodeUtf8(q.substring(pos,p1));
					value = decodeUtf8(q.substring(p1+1,p2));
					pos = p2+1;
				}
			}
			
			List<String> ov = ret.get(key);
			
			if (ov == null) {
				ov = new ArrayList<String>();
				ret.put(key,ov);
			}
			ov.add(value);
			
		} while (pos < q.length());
		
		return ret;
	}
}
