/***********************************************************
 * $Id: HttpLoginServiceRegistry.java 528 2011-09-04 14:37:35Z util $
 * 
 * HTTP Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 15.08.2011
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.http.sec;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * <p>This class may be used as a service listener, which keeps track
 * of all registered {@link HttpLoginService} instance.</p>
 * 
 * <p>This class should be registered as a blueprint bean like follows:</p>
 * <pre>
 * &lt;bp:reference-list id="loginServiceList" availability="optional" interface="org.clazzes.util.http.sec.HttpLoginService"&gt;
 *    &lt;bp:reference-listener ref="loginServiceRegistry" bind-method="loginServiceBound" unbind-method="loginServiceUnbound"/&gt;
 * &lt;/bp:reference-list&gt;
 *
 * &lt;bp:bean id="loginServiceRegistry" class="org.clazzes.util.http.sec.HttpLoginServiceRegistry"/&gt;
 * </pre>
 */
public class HttpLoginServiceRegistry {

	private static final Logger log = LoggerFactory.getLogger(HttpLoginServiceRegistry.class);
	
	private final ConcurrentMap<String,HttpLoginService> loginServicesByMethod;
	
	public HttpLoginServiceRegistry() {
		this.loginServicesByMethod = new ConcurrentHashMap<String,HttpLoginService>();
	}
	
	public HttpLoginService getLoginService(String method) {
		return this.loginServicesByMethod.get(method);
	}
	
	/**
	 * An OSGi <code>bind-method</code> for a HttpLoginService.
	 * @param svc The service becoming available.
	 * @param props The service properties, which should contain a
	 *              <code>login.mechanism</code> property.
	 */
	public void loginServiceBound(HttpLoginService svc, Map<String,String> props) {

		if (log.isDebugEnabled()) {
			log.debug("urlEditorBound: svc=[{}],props=[{}].",svc,props);
		}
		
		if (svc == null) return;
		
		String mechanism = props.get(HttpLoginService.LOGIN_MECHANISM_KEY);
		
		if (mechanism == null) {
			log.warn("No ["+HttpLoginService.LOGIN_MECHANISM_KEY+"] property in HttpLoginService when binding service.");
			return;
		}

		HttpLoginService entry = this.loginServicesByMethod.putIfAbsent(mechanism,svc);
		
		if (entry != null) {
		
			log.warn("Another HttpLoginService has been registered for mechanism [{}], skipping registration.",mechanism);
		}
	}
	
	/**
	 * An OSGi <code>unbind-method</code> for a HttpLoginService.
	 * @param svc The service becoming unavailable.
	 * @param props The service properties, which should contain a
	 *              <code>login.mechanism</code> property.
	 */
	public void loginServiceUnbound(HttpLoginService svc, Map<String,String> props) {
		
		if (log.isDebugEnabled()) {
			log.debug("urlEditorUnbound: svc=[{}],props=[{}].",svc,props);
		}
		
		if (svc == null) return;
		
		String mechanism = props.get(HttpLoginService.LOGIN_MECHANISM_KEY);
		
		if (mechanism == null) {
			log.warn("No ["+HttpLoginService.LOGIN_MECHANISM_KEY+"] property in HttpLoginService when unbinding service.");
			return;
		}

		boolean entry = this.loginServicesByMethod.remove(mechanism,svc);
		
		if (!entry) {
			log.warn("Unregistered HttpLoginService for role [{}] when unbinding service.",mechanism);
		}
	}
}
